function freqImage = BeatFrequencyFFTMapping
%% Variables to set before running analysis 
    % Defining Frame rate of Image taken
        ExposureTime = 100;%Exposure time in ms
        FrameRate = 1/(ExposureTime/1000); % This is in frames per second 
        %FrameRate = 10; % This is in frames per second 
        TimeScale = (1/FrameRate)*1000; % Time in milliseconds per frame

    %Choosing the frame range, use imageJ to select frames. If you're doing
    %an ROI analysis only you can use all frames. If you're doing a full
    %analysis then use imageJ to find these. The first frame should be at
    %the very beginning of a calcium wave. The last frame can either be the
    %last recorded frame or can be used to crop the video if there is
    %motion artifact. Several peaks are required for the full analysis code
    %to work
        FullImageStackLength = 100;
        FrameRangeStart = 1;
        FrameRangeEnd = 90;        
        
        
    % Setting the Final Image Scale
        TotalMag = 14.4; % This is the total maginification of the sytem during imaging 
        %CameraPixelSize = 20; % This is the size of a pixel for the camera in microns 
        CameraPixelSize = 11; % This is the size of a pixel for the camera in microns 
        MicronPerPixel = CameraPixelSize/TotalMag; 
        NumTickMarks= 10;%#X and Y Ticks
        NumBarTicks = 10;%#Colorbar Ticks
    
           %FillWindow for outlier removal USE ONLY ODD NUMBERS, larger numbers
   %will smooth the final calcium wave images more usual range (5-41)
        fillWindowCorrelation = 3;
        
    %How much to downsample the image for analysis
        downSampling = 7;
        
   %Sevitsky golay smoothing for cross correlation (default sevGolWindw =
   %11, sevGolOrder = 3)
        sevGolWindow = 5;
        sevGolOrder = 3;
           
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Number of images set by the start and end frame
num_images = FrameRangeEnd-FrameRangeStart+1;

%Read in the images from user input
%Let the user know that they should select an image
prompt1 = {'Select the images of interest'};
dlg_title1 = 'Select the images of interest';
num_lines = 0;
options.WindowStyle = 'normal';
answer = inputdlg(prompt1,dlg_title1,num_lines,{''},options);

[FileName,PathName] = uigetfile('.tif','Select the File to Image');
[ImageStack, FileName, PathName, ColorMapFrame, endframe ] = StackImport( '*.tif', '', 1, 1,FileName,PathName,FullImageStackLength);

%Pull out the first image and subtract it from the rest of the image stack
ImageStack = ImageStack(:,:,FrameRangeStart:FrameRangeEnd);

[convImageStack,xsampling,ysampling] = cropAndBlurFullAnalysis(downSampling,ImageStack, num_images);

[intensityTraces,downSampleRowSize,downSampleColSize] = downSampleFullAnalysis(convImageStack,ysampling,xsampling,num_images);

% figure
% imshow(intensityTraces(:,:,1),[])
% pause

[rDown,cDown,time] = size(intensityTraces);

freqImage = zeros(rDown,cDown);
smoothedTraces = zeros(rDown,cDown,time);
smoothedDCRemovedTraces = zeros(rDown,cDown,time);

parfor i = 1:rDown
    for j = 1:cDown
        currentTrace = intensityTraces(i,j,:);
        currentTrace = currentTrace(:);
        
        %Smooth the intensity traces using a third order savitzky golay
        %filter with a window of 21 samples and store the result
        smoothedTrace = sgolayfilt(currentTrace,sevGolOrder,sevGolWindow);
        smoothedTraces(i,j,:) = smoothedTrace;
        
        %Normailze the smoothed trace by subtracting the mean and dividing
        %by the max of the absolute value of the trace.
        smoothedDCRemovedTrace = (smoothedTrace-mean(smoothedTrace));
        smoothedDCRemovedTraces(i,j,:) = smoothedDCRemovedTrace;
        
%         figure
%         plot(smoothedDCRemovedTrace)
%         pause
        
        
        Y = fft(smoothedDCRemovedTrace);
        P2 = abs(Y/FullImageStackLength);
        P1 = P2(1:floor(FullImageStackLength/2+1));
        P1(2:end-1) = 2*P1(2:end-1);
        f = FrameRate*(0:(FullImageStackLength/2))/FullImageStackLength;
                
%         figure
%         plot(f,P1) 
        
        [pk,MaxFreq] = findpeaks(P1,'NPeaks',1,'SortStr','descend');
        if(numel(f(MaxFreq))~=0)
            freqImage(i,j) = f(MaxFreq);
        end
        
%         hold on
%         title('Single-Sided Amplitude Spectrum of X(t)')
%         xlabel('f (Hz)')
%         ylabel('|P1(f)|')
%         plot(f(MaxFreq),pk,'*r')
%         pause
%         close all
    end
end

figure
imshow(freqImage,'Colormap',jet)

% FFTFilled = fillImage(freqImage, fillWindowCorrelation, 'linear', 'quartiles');
FFTFilled = freqImage;

figure
imshow(FFTFilled,'Colormap',jet)

[FFTFilledScaled,ngridUpsample2X,ngridUpsample2Y,correlationImageRow,correlationImageCol,finalRow, finalCol,ngridInterpolant2X,ngridInterpolant2Y] = upscaleFullAnalysis(FFTFilled,ysampling,xsampling);

figure
imshow(FFTFilledScaled,'Colormap',jet)

%Masking the Image to the Mesh Pattern 
%Read in the images from user input
%Let the user know that they should select an mask
prompt1 = {'Select the image mask'};
dlg_title1 = 'Select the image mask';
num_lines = 0;
options.WindowStyle = 'normal';
answer = inputdlg(prompt1,dlg_title1,num_lines,{''},options);
[MaskFileName,MaskPathName] = uigetfile('.tif','Select the mask');
maskImage = imread(MaskFileName);
    maskImage = double(maskImage);

[r_maskImage, c_maskImage] = size(maskImage);


ScaledFFTImage = imresize(FFTFilledScaled,[r_maskImage c_maskImage],'bilinear');

MaskedFFTFilledScaled = ScaledFFTImage .* maskImage; % Multiplying the FFTFiledScaled image by the mask to remove the unwanted data

UniqueFrequencies = unique(MaskedFFTFilledScaled);
% [BinnedFrequencies,BinEdges] = histcounts(UniqueFrequencies,10);

figure('Name','FFT Histogram','NumberTitle','off')
FFThistogram = histogram(UniqueFrequencies,10);
%[correlationFilledScaled,correlationFilledScaledTime,ngridUpsample2X,ngridUpsample2Y,correlationImageRow,correlationImageCol,finalRow, finalCol,ngridInterpolant2X,ngridInterpolant2Y] = upscaleFullAnalysis(FFTStuff,ysampling,xsampling,TimeScale)

figure('Name','FFT Map','NumberTitle','off')

        % Shows a range from Black to Gray to Red in RGB Values (0,0,0->135,135,135->255,0,0)
        BGRColorMap = [0,0,0;0.0340557275529412,0.0340557275529412,0.0340557275529412;0.0681114550980392,0.0681114550980392,0.0681114550980392;0.102167182666667,0.102167182666667,0.102167182666667;0.136222910235294,0.136222910235294,0.136222910235294;0.170278637764706,0.170278637764706,0.170278637764706;0.204334365333333,0.204334365333333,0.204334365333333;0.238390092862745,0.238390092862745,0.238390092862745;0.272445820431373,0.272445820431373,0.272445820431373;0.306501548000000,0.306501548000000,0.306501548000000;0.340557275529412,0.340557275529412,0.340557275529412;0.374613003098039,0.374613003098039,0.374613003098039;0.408668730588235,0.408668730588235,0.408668730588235;0.442724458039216,0.442724458039216,0.442724458039216;0.476780185882353,0.476780185882353,0.476780185882353;0.510835913333333,0.510835913333333,0.510835913333333;0.544891640784314,0.544891640784314,0.544891640784314;0.578947368235294,0.578947368235294,0.578947368235294;0.613003096078431,0.613003096078431,0.613003096078431;0.647058823529412,0.647058823529412,0.647058823529412;0.655080213725490,0.632352941176471,0.632352941176471;0.663101604313726,0.617647058823529,0.617647058823529;0.671122994509804,0.602941176470588,0.602941176470588;0.679144385098039,0.588235294117647,0.588235294117647;0.687165775294118,0.573529411764706,0.573529411764706;0.695187165882353,0.558823529411765,0.558823529411765;0.703208556078431,0.544117647058824,0.544117647058824;0.711229946666667,0.529411764705882,0.529411764705882;0.719251336862745,0.514705882352941,0.514705882352941;0.727272727450980,0.500000000000000,0.500000000000000;0.735294117647059,0.485294117647059,0.485294117647059;0.743315507843137,0.470588235294118,0.470588235294118;0.751336898431373,0.455882352941176,0.455882352941176;0.759358288627451,0.441176470588235,0.441176470588235;0.767379679215686,0.426470588235294,0.426470588235294;0.775401069411765,0.411764705882353,0.411764705882353;0.783422460000000,0.397058823529412,0.397058823529412;0.791443850196078,0.382352941176471,0.382352941176471;0.799465240784314,0.367647058823529,0.367647058823529;0.807486630980392,0.352941176470588,0.352941176470588;0.815508021568628,0.338235294117647,0.338235294117647;0.823529411764706,0.323529411764706,0.323529411764706;0.831550801960784,0.308823529411765,0.308823529411765;0.839572192549020,0.294117647058824,0.294117647058824;0.847593582745098,0.279411764705882,0.279411764705882;0.855614973333333,0.264705882352941,0.264705882352941;0.863636363529412,0.250000000000000,0.250000000000000;0.871657754117647,0.235294117647059,0.235294117647059;0.879679144313725,0.220588235294118,0.220588235294118;0.887700534901961,0.205882352941176,0.205882352941176;0.895721925098039,0.191176470588235,0.191176470588235;0.903743315686275,0.176470588235294,0.176470588235294;0.911764705882353,0.161764705882353,0.161764705882353;0.919786096078431,0.147058823529412,0.147058823529412;0.927807486666667,0.132352941176471,0.132352941176471;0.935828876862745,0.117647058823529,0.117647058823529;0.943850267450980,0.102941176470588,0.102941176470588;0.951871657647059,0.0882352941176471,0.0882352941176471;0.959893048235294,0.0735294117647059,0.0735294117647059;0.967914438431373,0.0588235294117647,0.0588235294117647;0.975935829019608,0.0441176470588235,0.0441176470588235;0.983957219215686,0.0294117647058824,0.0294117647058824;0.991978609803922,0.0147058823529412,0.0147058823529412;1,0,0];

        FinalMap = imagesc(MaskedFFTFilledScaled);

        colormap(BGRColorMap) % Setting the color map to display custom LUT and editing the ColorBar. Must be on a range from 0-1 so divide by 255.
            cbar = colorbar;
            % Creating the cBar Ticks interval based upon defined increment and
            % range 
            Limits = cbar.Limits; % Getting the limits of the cbar 
                LowLim = Limits(1,1);
                HighLim = Limits(1,2);
            TickInterval = floor(HighLim/NumBarTicks); % This is a user defined input
                if TickInterval ==0
                    TickInterval = 0.25;
                end
            cBarTickMarks = zeros(1,NumBarTicks+1); % Creating the Matrix for use in manual tick assignment, +1 is needed so that this series starts at 0  
                for i = 1:NumBarTicks
                    cBarTickMarks(1,i+1) = TickInterval*i;
                end
            cbar.Ticks = cBarTickMarks;
            cbar.Label.String = 'Frequency (Hz)';
            cbar.Label.FontSize = 12;
            cbar.FontSize = 10;
            cbar.LineWidth = 1;

     % Adjusting the Axis labels to be in microns based upon the imaging settings used during acquisition         
        ax = gca;
        [r,c] = size(MaskedFFTFilledScaled);
        ax.PlotBoxAspectRatio = [1, r/c, 1];
        dimXY = size(MaskedFFTFilledScaled); % Getting the size of the XY dimension 
            XTickLength = dimXY(1,2); % This value is in pixels
            YTickLength = dimXY(1,1); % This value is in pixels    
        ax.XTick = 0:(XTickLength/NumTickMarks):XTickLength; % Defining the number of tick marks and their location 
        ax.YTick = 0:(YTickLength/NumTickMarks):XTickLength; % Defining the number of tick marks and their location 

        % Labeling the tick marks with microns units 
        LabelIncrementX = XTickLength/NumTickMarks;
        LabelIncrementY = YTickLength/NumTickMarks;
            XlabelString = zeros(1,NumTickMarks+1);
                for i = 1:NumTickMarks
                    CurrentString = [(LabelIncrementX*i)*MicronPerPixel];
                    XlabelString(1,i+1) = CurrentString/1000;%From micron to mm
                end
            ax.XTickLabel = string(round(XlabelString,1)); % Setting the X label to be in microns and rounding, Can change the rounding 
            ax.XLabel.String = 'Distance (mm)'; % Setting the name of the X-axis 
            ax.XLabel.FontSize = 12; % Setting the font size of the X-axis 

            YlabelString = zeros(1,NumTickMarks+1);
                for i = 1:NumTickMarks
                    CurrentString = [(LabelIncrementY*i)*MicronPerPixel];
                    YlabelString(1,i+1) = CurrentString/1000;%From micron to mm
                end
            YlabelString = rot90(YlabelString,2);    
            ax.YTickLabel = string(round(YlabelString,1)); % Setting the X label to be in microns and rounding, Can change the rounding
            ax.YLabel.String = 'Distance (mm)'; % Setting the name of the X-axis 
            ax.YLabel.FontSize = 12; % Setting the font size of the X-axis 
                %ax.YTickLabel = {}; % This will set the Y label to blank 

        % Creating the Title for the Figure     
        ax.Title.String = 'FFT Image Frequency Map'; % Title text 
        ax.Title.FontWeight = 'normal'; % This is where you would change to bold etc. if wanted 
        ax.Title.FontSize = 14; % Defining the Font size for the Title 
        ax.LineWidth = 1;

        
        TopFrequencies = [];
        NumDataPoints = 2;
        for i = 1:NumDataPoints
            set(gcf,'CurrentCharacter',char(1));
            h=datacursormode;
            set(h,'DisplayStyle','datatip','SnapToData','off');
            waitfor(gcf,'CurrentCharacter',char(32)); % Need to hit space bar to collect data and make the table
            s = getCursorInfo(h);
            x = s.Position;
            TopFrequencies(i,:)= s.Target.CData(x(1,2),x(1,1));
        end
        
        figure('Name','FFT DataPoints Table','NumberTitle','off')
        FrequencyTable = table(TopFrequencies);
        uitable('Data',FrequencyTable{:,:},'ColumnName',FrequencyTable.Properties.VariableNames,...
        'RowName',FrequencyTable.Properties.RowNames,'Units', 'Normalized', 'Position',[0, 0, 1, 1]);

end

%This function imports a tiff stack
function [ImageStack, FileName, PathName, ColorMapFrame, endframe ] = StackImport(file_format, directory, files, ColorMapFrame,FileName,PathName, num_images)

%Importing a Tiff Image Stack for Analysis 
AnalysisFile = fullfile(PathName,FileName); %Constructs a variable containing the file path and the file name 

numfiles = files; %Used to idicate how many files are selected 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Creating the file directory for processing 

filedirs = cell(1,numfiles);
filedirs{1} = AnalysisFile;

filenum = size(filedirs,2);
filedir = filedirs{1};

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Getting the image file information 

fprintf('\nAcquiring Information for Image File # %d ...\n\n',filenum);

% This should work for image stacks as well 
if ~exist('IS','var')
    info = imfinfo(filedir);
    num_images = numel(info);
    
    parfor k = 1:num_images
        %fprintf('\nreading frame : %d\n', k);
        IS2(:,:,k) = imread(filedir, k, 'Info', info);
        
    end
end

fprintf('\nImage File Information Obtained ...\n');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Creating Image Stack

startframe = 1;
endframe = num_images;

Stack = IS2(:,:,startframe:endframe);
ImageStack = Stack;

end

%This function crops the image stack and blurs it prior to analysis
function [convImageStack,xsampling,ysampling] = cropAndBlurFullAnalysis(downSampling,ImageStack, num_images)
%Determine undersampling in x, and sigma
    xsampling = downSampling;
    xsigma = xsampling/3;
    numPixelsX = 2*ceil(3*xsigma)+1;

    %Determine undersampling in y, and sigma
    ysampling = downSampling;
    ysigma = ysampling/3;
    numPixelsY = 2*ceil(3*ysigma)+1;

    %Preallocate filter vector
    gaussianX = zeros(1,numPixelsX);
    gaussianY = zeros(1,numPixelsY);

    %Fill the Xgaussian filter vector
    parfor i = 1:numPixelsX
       startPos = i-1.5-ceil(3*xsigma);%3*sigma pixels to the left of center
       endPos = i-.5-ceil(3*xsigma);%3*sigma pixels to the right of center
       gaussianX(i) = trapz(normpdf(startPos:.001:endPos,0,xsigma)); %unnormalized gaussian
    end

    %Fill the Ygaussian filter vector
    parfor i = 1:numPixelsY
       startPos = i-1.5-ceil(3*ysigma);%3*sigma pixels to the left of center
       endPos = i-.5-ceil(3*ysigma);%3*sigma pixels to the right of center
       gaussianY(i) = trapz(normpdf(startPos:.001:endPos,0,ysigma)); %unnormalized gaussian
    end

    %normalize the gaussian
    gaussianX = gaussianX./sum(gaussianX);
    gaussianY = gaussianY./sum(gaussianY);

    %convolve the vectors with the image to get the hessian indices at each
    %point
    convImageStack = zeros(size(ImageStack));
    parfor i = 1:num_images
        convImageStack(:,:,i) = conv2(gaussianY',gaussianX,ImageStack(:,:,i),'same');
    end
end

%Downsamples cropped image stack prior to analysis
function [intensityTraces,downSampleRowSize,downSampleColSize] = downSampleFullAnalysis(cropConvImageStack,ysampling,xsampling,num_images)
%Generate an interpolant of the whole image
    [convolutionCroppedRows,convolutionCroppedCol,n] = size(cropConvImageStack(:,:,1));
    [ngridInterpolant1X,ngridInterpolant1Y] = ndgrid(1:convolutionCroppedRows,1:convolutionCroppedCol);

    %Describe the locations of interest for the undersampling of the whole
    %image
    [ngridDownsample1X,ngridDownsample1Y] = ndgrid(1:ysampling:convolutionCroppedRows,1:xsampling:convolutionCroppedCol);%Ngrid of the undersampled image
    [downSampleRowSize,downSampleColSize] = size(ngridDownsample1X);%Size of the undersampled image
    intensityTraces = zeros(downSampleRowSize,downSampleColSize,num_images);
    
    %Store the undersampled images
    parfor i = 1:num_images
        currentImInterpolant = griddedInterpolant(ngridInterpolant1X,ngridInterpolant1Y,double((cropConvImageStack(:,:,i))),'cubic');
        vq = currentImInterpolant(ngridDownsample1X,ngridDownsample1Y);
        intensityTraces(:,:,i) = vq;
    end
end

%Runs through the pixels and replaces outliers depending on a surrounding
%box and the built in function filloutliers Use only odd box sizes
function filledImage = fillImage(image, boxsize, fillMethod, outlierMethod)
    [downSampleRowSize,downSampleColSize] = size(image);
    left = (boxsize-1)/2;
    filledImage = image;
        parfor i = 1:downSampleRowSize
            for j = 1:downSampleColSize
                    currentLeft = max([j-left, 1]);
                    currentRight = min([j+left, downSampleColSize]);
                    currentTop = max([i-left, 1]);
                    currentBottom = min([i+left, downSampleRowSize]);

                    %Location in current box where the center of the moving
                    %window is
                    if(currentLeft == 1)
                        centerCol = j;
                    else
                        centerCol = left+1;
                    end

                    if(currentTop == 1)
                        centerRow = i;
                    else
                        centerRow = left+1;
                    end

                    width = currentRight-currentLeft+1;
                    height = currentBottom-currentTop+1;

                    %Pull the pixels in the window and reshape into a
                    %vector
                    currentBox = image(currentTop:currentBottom,currentLeft:currentRight);
                    currentBox = currentBox(:);
                    
                    %Fill the outliers in the vector and reshape into the
                    %window
                    firstFill = filloutliers(currentBox,fillMethod,outlierMethod);
                    firstFill = reshape(firstFill,[height,width]);

                    %Replace the value in the center of the window with the
                    %value from the filled window
                    filledImage(i,j)=firstFill(centerRow,centerCol);
    %                 imagesc()
    %                 pause
            end
        end
end

%This function returns the analysis to the original scale
function [FFTFilledScaled,ngridUpsample2X,ngridUpsample2Y,correlationImageRow,correlationImageCol,finalRow, finalCol,ngridInterpolant2X,ngridInterpolant2Y] = upscaleFullAnalysis(FFTFilled,ysampling,xsampling)
    %Generate an interpolant of the whole image
    [correlationImageRow,correlationImageCol] = size(FFTFilled);
    [ngridInterpolant2X,ngridInterpolant2Y] = ndgrid(1:correlationImageRow,1:correlationImageCol);

    %Describe the locations of interest for the oversample of the whole
    %image
    [ngridUpsample2X,ngridUpsample2Y] = ndgrid(1:(1/ysampling):correlationImageRow,1:(1/xsampling):correlationImageCol);

    %Store the oversampled images
    currentImInterpolant = griddedInterpolant(ngridInterpolant2X,ngridInterpolant2Y,double(FFTFilled),'linear');
    vq = currentImInterpolant(ngridUpsample2X,ngridUpsample2Y);
    FFTFilledScaled = vq;

    %Get the size of the correlation matrix
    [finalRow, finalCol] = size(FFTFilledScaled);
end
























